from typing import List, Dict
from braintree.graphql.inputs.customer_session_input import CustomerSessionInput
from braintree.graphql.enums.recommendations import Recommendations


class CustomerRecommendationsInput:
    """
    Represents the input to request PayPal customer session recommendations.
    """

    def __init__(
        self,
        session_id: str,
        recommendations: List[Recommendations],
        merchant_account_id: str = None,
        customer: CustomerSessionInput = None,
    ):
        self._session_id = session_id
        self._recommendations = recommendations
        self._merchant_account_id = merchant_account_id
        self._customer = customer

    def to_graphql_variables(self) -> Dict:
        variables = {
            "sessionId": self._session_id,
            "recommendations": [recommendation.name for recommendation in self._recommendations],
        }
        if self._merchant_account_id is not None:
            variables["merchantAccountId"] = self._merchant_account_id
        if self._customer is not None:
            variables["customer"] = (
                self._customer.to_graphql_variables()
                if hasattr(self._customer, "to_graphql_variables")
                else self._customer
            )
        return variables

    @staticmethod
    def builder(session_id: str, recommendations: List[Recommendations]):
        """
        Creates a builder instance for fluent construction of CustomerRecommendationsInput objects.

        Args:
            session_id (str): ID of the customer session for getting recommendations.
            recommendations (List[Recommendations]): List of recommendations to retrieve for the customer session.
        """
        return CustomerRecommendationsInput.Builder(session_id, recommendations)

    class Builder:
        def __init__(self, session_id: str, recommendations: List[Recommendations]):
            self._session_id = session_id
            self._recommendations = recommendations
            self._merchant_account_id = None
            self._customer = None

        def merchant_account_id(self, merchant_account_id: str):
            """
            Sets the merchant account ID.
            """
            self._merchant_account_id = merchant_account_id
            return self

        def customer(self, customer: CustomerSessionInput):
            """
            Sets the input object representing customer information relevant to the customer session.
            """
            self._customer = customer
            return self

        def build(self):
            return CustomerRecommendationsInput(
                self._session_id,
                self._recommendations,
                self._merchant_account_id,
                self._customer,
            )



from typing import List
from braintree.graphql.types.payment_options import PaymentOptions


class CustomerRecommendations:
    """
    A union of all possible customer recommendations associated with a PayPal customer session.
    """

    def __init__(self, payment_options: List[PaymentOptions]):
        self.payment_options = payment_options



from typing import Dict

from braintree.graphql.inputs.phone_input import PhoneInput


class CustomerSessionInput:
    """
    Customer identifying information for a PayPal customer session.
    """

    def __init__(
        self,
        email: str = None,
        phone: PhoneInput = None,
        device_fingerprint_id: str = None,
        paypal_app_installed: bool = None,
        venmo_app_installed: bool = None,
        user_agent: str = None,
    ):
        self._email = email
        self._phone = phone
        self._device_fingerprint_id = device_fingerprint_id
        self._paypal_app_installed = paypal_app_installed
        self._venmo_app_installed = venmo_app_installed
        self._user_agent = user_agent

    def to_graphql_variables(self) -> Dict:
        variables = {}
        if self._email is not None:
            variables["email"] = self._email
        if self._device_fingerprint_id is not None:
            variables["deviceFingerprintId"] = self._device_fingerprint_id
        if self._phone is not None:
            variables["phone"] = self._phone.to_graphql_variables()
        if self._paypal_app_installed is not None:
            variables["paypalAppInstalled"] = self._paypal_app_installed
        if self._venmo_app_installed is not None:
            variables["venmoAppInstalled"] = self._venmo_app_installed
        if self._user_agent is not None:
            variables["userAgent"] = self._user_agent

        return variables

    @staticmethod
    def builder():
        """
        Creates a builder instance for fluent construction of CustomerSessionInput objects.
        """
        return CustomerSessionInput.Builder()

    class Builder:
        def __init__(self):
            self._email = None
            self._phone = None
            self._device_fingerprint_id = None
            self._paypal_app_installed = None
            self._venmo_app_installed = None
            self._user_agent = None

        def email(self, email: str):
            """
            Sets the customer email address.
            """
            self._email = email
            return self

        def phone(self, phone: PhoneInput):
            """
            Sets the customer phone number input object.
            """
            self._phone = phone
            return self

        def device_fingerprint_id(self, device_fingerprint_id: str):
            """
            Sets the device fingerprint ID.
            """
            self._device_fingerprint_id = device_fingerprint_id
            return self

        def paypal_app_installed(self, paypal_app_installed: bool):
            """
            Sets whether the PayPal app is installed on the customer's device.
            """
            self._paypal_app_installed = paypal_app_installed
            return self

        def venmo_app_installed(self, venmo_app_installed: bool):
            """
            Sets whether the Venmo app is installed on the customer's device.
            """
            self._venmo_app_installed = venmo_app_installed
            return self

        def user_agent(self, user_agent: str):
            """
            Sets user agent from the request originating from the customer's device.
            This will be used to identify the customer's operating system and browser versions.
            """
            self._user_agent = user_agent
            return self

        def build(self):
            return CustomerSessionInput(
                self._email,
                self._phone,
                self._device_fingerprint_id,
                self._paypal_app_installed,
                self._venmo_app_installed,
                self._user_agent,
            )



from typing import Dict
from braintree.graphql.inputs.customer_session_input import CustomerSessionInput


class UpdateCustomerSessionInput:
    """
    Represents the input to request an update to a PayPal customer session.
    """

    def __init__(
        self,
        session_id: str,
        merchant_account_id: str = None,
        customer: CustomerSessionInput = None,
    ):
        self._session_id = session_id
        self._merchant_account_id = merchant_account_id
        self._customer = customer

    def to_graphql_variables(self) -> Dict:
        variables = {}
        if self._session_id:
            variables["sessionId"] = self._session_id
        if self._merchant_account_id:
            variables["merchantAccountId"] = self._merchant_account_id
        if self._customer:
            variables["customer"] = self._customer.to_graphql_variables()
        return variables

    @staticmethod
    def builder(session_id: str):
        """
        Creates a builder instance for fluent construction of UpdateCustomerSessionInput objects.

        Args:
            session_id (str): ID of the customer session to be updated.
        """
        return UpdateCustomerSessionInput.Builder(session_id)

    class Builder:
        def __init__(self, session_id: str):
            self._session_id = session_id
            self._merchant_account_id = None
            self._customer = None

        def merchant_account_id(self, merchant_account_id):
            """
            Sets the merchant account ID.
            """
            self._merchant_account_id = merchant_account_id
            return self

        def customer(self, customer):
            """
            Sets the input object representing customer information relevant to the customer session.
            """
            self._customer = customer
            return self

        def build(self):
            return UpdateCustomerSessionInput(
                self._session_id,
                self._merchant_account_id,
                self._customer
            )



from enum import Enum


class Recommendations(Enum):
    """
    Represents available types of customer recommendations that can be retrieved using a PayPal customer session.
    """

    PAYMENT_RECOMMENDATIONS = "PAYMENT_RECOMMENDATIONS"



from enum import Enum


class RecommendedPaymentOption(Enum):
    """
    Represents available payment options related to PayPal customer session recommendations.
    """

    PAYPAL = "PAYPAL"
    VENMO = "VENMO"



from braintree.graphql.unions.customer_recommendations import CustomerRecommendations


class CustomerRecommendationsPayload:
    """
    Represents the customer recommendations information associated with a PayPal customer session.
    """

    def __init__(self, is_in_paypal_network: bool, recommendations: CustomerRecommendations):
        self.is_in_paypal_network = is_in_paypal_network
        self.recommendations = recommendations
